' TODO: add type parameter to createlight
' TODO: LightConeAngles

Type LIGHT Extends ib3d_Entity
	Global LightList:TList=Null
	
	Method New()
		If Not LightList Then LightList=New TList
		LightList.AddLast(Self)
	EndMethod
	
	Method Destroy()
		Super.Destroy()
		LightList.Remove(Self)
	EndMethod	
	
   	Method setRange(range:Float=1000.0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Light node is invalid"
    	ILightSceneNode(_node).getLightData().setRadius(range)
   	EndMethod
    
   	Method SetColor(red:Float,green:Float,blue:Float,doNotUse:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Light node is invalid"
		setDiffuseColor(red,green,blue)
   	EndMethod

	Method setAmbientColor(red:Float,green:Float,blue:Float,doNotUse:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Light node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
       	red=colorf.getRed()
       	green=colorf.getGreen()
       	blue=colorf.getBlue()

   		ILightSceneNode(_node).getLightData().getAmbientColor().set(red,green,blue)
	EndMethod

	Method setDiffuseColor(red:Float,green:Float,blue:Float,doNotUse:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Light node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
       	red=colorf.getRed()
       	green=colorf.getGreen()
       	blue=colorf.getBlue()

   		ILightSceneNode(_node).getLightData().getDiffuseColor().set(red,green,blue)
	EndMethod
	
	Method setSpecularColor(red:Float,green:Float,blue:Float,doNotUse:Int=0)
		If Not _node Or Not _node.isValid() Then RuntimeError "Light node is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
       	red=colorf.getRed()
       	green=colorf.getGreen()
       	blue=colorf.getBlue()

   		ILightSceneNode(_node).getLightData().getSpecularColor().set(red,green,blue)
	EndMethod	

	Method setCastShadows(bCastShadows:Int)
		If Not _node Or Not _node.isValid() Then RuntimeError "Light node is invalid"
    		ILightSceneNode(_node).getLightData().setCastShadows(bCastShadows)
	EndMethod
			
EndType

Rem
- Light functions

Not Converted:
LightConeAngles 

Unsupported:
CreateLight parameter

EndRem

Rem
bbdoc: Creates a light
returns: A new instance of a light
about: <i>Parameters</i>
<ul>
<li>parent:ib3d_Entity - parent entity of light</li>
</ul>
The light will cast dynamic light on all other entities in the scene, which have the material flag EMF_LIGHTING turned on. (This is the default setting in most entities). 
EndRem
Function ib3d_CreateLight:LIGHT(parent:ib3d_Entity=Null)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.createLight(parent)
EndFunction

Rem
bbdoc: Sets the range of a light
about: <i>Parameters</i>
<ul>
<li>light:LIGHT - instance of a light</li>
<li>range:Float - range of light.  Default is 1000.0
</ul>
The range of a light is how far it reaches. Everything outside the range of the light will not be affected by it. 
EndRem
Function ib3d_LightRange(light:LIGHT,range:Float=1000.0)
	If light=Null Then Throw "Light does not exist"
	light.setRange(range)
EndFunction

Rem
bbdoc: Sets the color of a light
returns: <what it returns>
about: <i>Parameters</i>
<ul>
<li>light:LIGHT - instance of a light</li>
<li>red:Float - red value for light color</li>
<li>green:Float - green value for light color</li>
<li>blue:Float - blue value for light color</li>
</ul>
NOTE: negative values are not supported.
EndRem
Function ib3d_LightColor(light:LIGHT,red:Float,green:Float,blue:Float)
	If light=Null Then Throw "Light does not exist"
	light.setColor(red,green,blue)
EndFunction

Rem
bbdoc: Sets the ambient color for a light
about: <i>Parameters</i>
<ul>
<li>light:LIGHT - instance of a light</li>
<li>red:Float - red value for ambient color</li>
<li>green:Float - green value for ambient color</li>
<li>blue:Float - blue value for ambient color</li>
</ul>
Default is 0,0,0.
EndRem
Function ib3d_LightAmbientColor(light:LIGHT,red:Float,green:Float,blue:Float)
	If light=Null Then Throw "Light does not exist"
	light.setAmbientColor(red,green,blue)
EndFunction

Rem
bbdoc: Sets the diffuse color for a light
about: <i>Parameters</i>
<ul>
<li>light:LIGHT - instance of a light</li>
<li>red:Float - red value for diffuse color</li>
<li>green:Float - green value for diffuse color</li>
<li>blue:Float - blue value for diffuse color</li>
</ul>
Default is 255,255,255.
EndRem
Function ib3d_LightDiffuseColor(light:LIGHT,red:Float,green:Float,blue:Float)
	If light=Null Then Throw "Light does not exist"
	light.setDiffuseColor(red,green,blue)
EndFunction

Rem
bbdoc: Sets the specular color for a light
about: <i>Parameters</i>
<ul>
<li>light:LIGHT - instance of a light</li>
<li>red:Float - red value for specular color</li>
<li>green:Float - green value for specular color</li>
<li>blue:Float - blue value for specular color</li>
</ul>
Default is 255,255,255.
EndRem
Function ib3d_LightSpecularColor(light:LIGHT,red:Float,green:Float,blue:Float)
	If light=Null Then Throw "Light does not exist"
	light.setSpecularColor(red,green,blue)
EndFunction

Rem
bbdoc: Determines if the light is to cast shadows or not
about: <i>Parameters</i>
<ul>
<li>light:LIGHT - instance of a light</li>
<li>bCastShadows:Int - True if light is to cast shadows.  Default is True.</li>
</ul>
EndRem
Function ib3d_LightCastShadows(light:LIGHT,bCastShadows:Int)
	If light=Null Then Throw "Light does not exist"
	light.setCastShadows(bCastShadows)
EndFunction

