' ------------------------------------------------------------------------------------
' irr::core::aabbox3di wrappers
' ------------------------------------------------------------------------------------

Public

Type AABbox3di Extends IrrBase Final

	Method EqEq:Int(pOther:AABbox3di)
		Return IrrCore_AABbox3di_EqEq(handle,pOther.handle)
	EndMethod
	Method NotEq:Int(pOther:AABbox3di)
		Return IrrCore_AABbox3di_NotEq(handle,pOther.handle)
	EndMethod
	
	Method getEdges:Vector3di[]()
		Local edges:Vector3di[8]
		Local edgeparam:Int[8]		
		
		For Local i:Int=0 To 7
			edges[i]=Vector3di.create()
			edgeparam[i]=edges[i].handle
		Next

		IrrCore_AABbox3di_getEdges(handle,edgeparam)
		
		Return edges		
	EndMethod
	
	Method intersectsWithBox:Int(pOther:AABbox3di)
		Return IrrCore_AABbox3di_intersectsWithBox(handle,pOther.handle)
	EndMethod
	Method intersectsWithLine:Int(pLine:Line3df)
		Return IrrCore_AABbox3di_intersectsWithLine(handle,pLine.handle)
	EndMethod
	Method intersectsWithLineUsingVects:Int(pLineMiddle:Vector3di,pLineVect:Vector3di,halflength:Float)
		Return IrrCore_AABbox3di_intersectsWithLineUsingVects(handle,pLineMiddle.handle,pLineVect.handle,halflength)
	EndMethod
	Method classifyPlaneRelation:Int(pPlane:Plane3di)
		Return IrrCore_AABbox3di_classifyPlaneRelation(handle,pPlane.handle)
	EndMethod
	Method getCenter:Vector3di()
		Return Vector3di.createFromHandle(IrrCore_AABbox3di_getCenter(handle))
	EndMethod
	Method getExtent:Vector3di()
		Return Vector3di.createFromHandle(IrrCore_AABbox3di_getExtent(handle))
	EndMethod
	Method getMinEdge:Vector3di()
		' dont delete
		Return Vector3di.createFromHandle(IrrCore_AABbox3di_getMinEdge(handle),False)
	EndMethod
	Method getMaxEdge:Vector3di()
		' dont delete
		Return Vector3di.createFromHandle(IrrCore_AABbox3di_getMaxEdge(handle),False)
	EndMethod
	Method isEmpty:Int()
		Return IrrCore_AABbox3di_isEmpty(handle)
	EndMethod
	Method repair()
		IrrCore_AABbox3di_repair(handle)
	EndMethod	
	Method getInterpolated:AABbox3di(pOther:AABbox3di,d:Float)
		Return createFromHandle(IrrCore_AABbox3di_getInterpolated(handle,pOther.handle,d))
	EndMethod

	Method addInternalPoint(pPoint:Vector3di)
		IrrCore_AABbox3di_addInternalPoint(handle,pPoint.handle)
	EndMethod
	Method addInternalPointFromVals(x:Float,y:Float,z:Float)
		IrrCore_AABbox3di_addInternalPointFromVals(handle,x,y,z)
	EndMethod
	Method addInternalBox(pAddBox:AABbox3di)
		IrrCore_AABbox3di_addInternalBox(handle,pAddBox.handle)
	EndMethod
	Method resetFromVals(x:Float,y:Float,z:Float)
		IrrCore_AABbox3di_resetFromVals(handle,x,y,z)
	EndMethod
	Method resetFromBox(pInitValue:AABbox3di)
		IrrCore_AABbox3di_resetFromBox(handle,pInitValue.handle)
	EndMethod
	Method resetFromVect(pInitValue:Vector3di)
		IrrCore_AABbox3di_resetFromVect(handle,pInitValue.handle)
	EndMethod
	Method isPointInside:Int(pPoint:Vector3di)
		Return IrrCore_AABbox3di_isPointInside(handle,pPoint.handle)
	EndMethod
	Method isPointTotalInside:Int(pPoint:Vector3di)
		Return IrrCore_AABbox3di_isPointTotalInside(handle,pPoint.handle)
	EndMethod
	Method isFullInside:Int(other:AABbox3di)
		Return IrrCore_AABbox3di_isFullInside(handle,other.handle)
	EndMethod
	
	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
	
	Function create:AABbox3di()
		Return createFromHandle(IrrCore_AABbox3di_new())
	EndFunction
	
	Function createFromBox:AABbox3di(other:AABbox3di)
		Return createFromHandle(IrrCore_AABbox3di_newFromBox(other.handle))
	EndFunction

	Function createFromVect:AABbox3di(pInit:Vector3di)
		Return createFromHandle(IrrCore_AABbox3di_newFromVect(pInit.handle))
	EndFunction
	
	Function createFromVectWithMax:AABbox3di(pInit:Vector3di,pMax:Vector3di)
		Return createFromHandle(IrrCore_AABbox3di_newFromVectWithMax(pInit.handle,pMax.handle))
	EndFunction
	
	Function createFromVals:AABbox3di(minx:Float,miny:Float,minz:Float,maxx:Float,maxy:Float,maxz:Float)
		Return createFromHandle(IrrCore_AABbox3di_newFromVals(minx,miny,minz,maxx,maxy,maxz))
	EndFunction

	Function createFromHandle:AABbox3di(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:AABbox3di=New AABbox3di
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_AABbox3di_del(handle)
	EndMethod		
	
EndType