' ------------------------------------------------------------------------------------
' irr::core::line3d<s32> wrappers
' ------------------------------------------------------------------------------------
Public

Type Line3di Extends IrrBase Final

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------

	Method Plus:Line3di(pPoint:Vector3di)
		Return createFromHandle(IrrCore_Line3di_Plus(handle,pPoint.handle))
	EndMethod
	Method Minus:Line3di(pPoint:Vector3di)
		Return createFromHandle(IrrCore_Line3di_Minus(handle,pPoint.handle))
	EndMethod
	Method PlusEq(pPoint:Vector3di)
		IrrCore_Line3di_PlusEq(handle,pPoint.handle)
	EndMethod
	Method MinusEq(pPoint:Vector3di)
		IrrCore_Line3di_PlusEq(handle,pPoint.handle)
	EndMethod
	Method EqEq:Int(pLine2:Line3di)
		Return IrrCore_Line3di_EqEq(handle,pLine2.handle)
	EndMethod
	Method NotEq:Int(pLine2:Line3di)
		Return IrrCore_Line3di_NotEq(handle,pLine2.handle)
	EndMethod

	'--------------------------------------------
	' get/set routines
	'--------------------------------------------
	
	Method setLineFromVals(xa:Float,ya:Float,za:Float,xb:Float,yb:Float,zb:Float)
		IrrCore_Line3di_setLineFromVals(handle,xa,ya,za,xb,yb,zb)
	EndMethod
	
	Method setLineFromVects(pStart:Vector3di,pEnd:Vector3di)
		IrrCore_Line3di_setLineFromVects(handle,pStart.handle,pEnd.handle)
	EndMethod
	
	Method setLineFromLine(pLine2:Line3di)
		IrrCore_Line3di_setLineFromLine(handle,pLine2.handle)
	EndMethod

	Method getLength:Double()
		Return IrrCore_Line3di_getLength(handle)
	EndMethod
	Method getLengthSQ:Int()
		Return IrrCore_Line3di_getLengthSQ(handle)
	EndMethod

	Method getMiddle:Vector3di()
		Return Vector3di.createFromHandle(IrrCore_Line3di_getMiddle(handle))
	EndMethod
	
	Method getVector:Vector3di()
		Return Vector3di.createFromHandle(IrrCore_Line3di_getVector(handle))
	EndMethod
	Method isPointBetweenStartAndEnd:Int(pPoint:Vector3di)
		Return IrrCore_Line3di_isPointBetweenStartAndEnd(handle,pPoint.handle)
	EndMethod

	Method getClosestPoint:Vector3di(pPoint:Vector3di)
		Return Vector3di.createFromHandle(IrrCore_Line3di_getClosestPoint(handle,pPoint.handle))
	EndMethod
	
	Method getStart:Vector3di()
		' set to not delete
		Return Vector3di.createFromHandle(IrrCore_Line3di_getStart(handle),False)
	EndMethod
	Method getEnd:Vector3di()
		' set to not delete
		Return Vector3di.createFromHandle(IrrCore_Line3di_getEnd(handle),False)
	EndMethod

	Method getIntersectionWithSphere:Int(pSorigin:Vector3di,sradius:Float,pOutdistance:Double Ptr)
		Return IrrCore_Line3di_getIntersectionWithSphere(handle,pSorigin.handle,sradius,pOutdistance)
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
	
	Function createFromVals:Line3di(xa:Float,ya:Float,za:Float,xb:Float,yb:Float,zb:Float)
		Return createFromHandle(IrrCore_Line3di_newFromVals(xa,ya,za,xb,yb,zb))
	EndFunction

	Function createFromVects:Line3di(pStart:Vector3di,pEnd:Vector3di)
		Return createFromHandle(IrrCore_Line3di_newFromVects(pStart.handle,pEnd.handle))
	EndFunction

	Function createFromLine:Line3di(pOther:Line3di)
		Return createFromHandle(IrrCore_Line3di_newFromLine(pOther.handle))
	EndFunction

	Function create:Line3di()
		Return createFromHandle(IrrCore_Line3di_new())
	EndFunction
	
	Function createFromHandle:Line3di(pLine:Int,bMustDel:Int=True)
		If Not pLine Then Return Null
		Local oRetVal:Line3di=New Line3di
		oRetVal.attach(pLine,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Line3di_del(handle)
	EndMethod	
EndType


