' ------------------------------------------------------------------------------------
' irr::scene::IParticleAffector wrappers
' TODO: implement serialize/deserialize on default type when fixed
' ------------------------------------------------------------------------------------
Public

Type IParticleAffector Extends IAttributeExchangingObject

	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------

	Method affect(now:Int,particles:Array_SParticle) Abstract

	Method serializeAttributes(out:IAttributes,options:SAttributeReadWriteOptions=Null)	Abstract

	Method deserializeAttributes(in:IAttributes,options:SAttributeReadWriteOptions=Null) Abstract

	Method getType:Int() Abstract

	' --------------------------------------------------------------------------------
	' default methods
	' --------------------------------------------------------------------------------

	Method setEnabled(bEnabled:Int)
		IrrScene_IParticleAffector_setEnabled(handle,bEnabled)
	EndMethod
	
	Method getEnabled:Int()
		Return IrrScene_IParticleAffector_getEnabled(handle)
	EndMethod

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_IParticleAffector_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_IParticleAffector_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_IParticleAffector_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:IParticleAffector(pGenFunc:Byte Ptr) Final
		Return createFromHandle(IrrScene_BMXIParticleAffector_new(),True,pGenFunc)
	EndFunction

	Function createFromHandle:IParticleAffector(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr=Null) Final
		If Not pHandle Then Return Null
		Local oRetVal:IParticleAffector=Null
		
		If pGenFunc
			' use the funtion pointer to generate
			Local generate:IParticleAffector()=pGenFunc
		
			oRetVal=generate()
		Else
			oRetVal=New IParticleAffectorDefault
		EndIf

		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound=bMustDel And pGenFunc
		
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal
	EndFunction

	' --------------------------------------------------------------------------------
	' bindings
	' --------------------------------------------------------------------------------	

	Method bind(rec:Int) Final
		Super.bind(rec)
		IrrScene_BMXIParticleAffector_bindAffect( ..
			translate(rec).handle, ..
			OnAffectBinding)
		IrrScene_BMXIParticleAffector_bindSerializeAttributes( ..
			translate(rec).handle, ..
			OnSerializeAttributes)
		IrrScene_BMXIParticleAffector_bindDeserializeAttributes( ..
			translate(rec).handle, ..
			OnDeserializeAttributes)
		IrrScene_BMXIParticleAffector_bindGetType( ..
			translate(rec).handle, ..
			OnGetType)			
	EndMethod

	Function OnGetType:Int(binding:Int)
		' get the object whose node is firing
		Local tempobj:IParticleAffector=IParticleAffector(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.getType()
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function OnDeserializeAttributes:Int(binding:Int,startIndex:Int,pIAttributes:Int,pSAttributeReadWriteOptions:Int)
		' get the object whose node is firing
		Local tempobj:IParticleAffector=IParticleAffector(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		Return tempobj.deserializeAttributes( ..
				IAttributes.createFromHandle(pIAttributes,False), ..
				SAttributeReadWriteOptions.createFromHandle(pSAttributeReadWriteOptions))
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Function OnSerializeAttributes(binding:Int,pIAttributes:Int,pSAttributeReadWriteOptions:Int)
		' get the object whose node is firing
		Local tempobj:IParticleAffector=IParticleAffector(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		tempobj.serializeAttributes( ..
				IAttributes.createFromHandle(pIAttributes,False), ..
				SAttributeReadWriteOptions.createFromHandle(pSAttributeReadWriteOptions))
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction

	Function OnAffectBinding(binding:Int,now:Int,pArray_SParticle:Int) Final
		' get the object whose node is firing
		Local tempobj:IParticleAffector=IParticleAffector(translate(binding))
		
		If tempobj<>Null
	  		' get the particle array.  be sure to delete when done.
  			Local particles:Array_SParticle=Array_SParticle.createFromHandle(pArray_SParticle,True)
  			' fire the event
	  		tempobj.affect(now,particles)
		Else
			Throw "Unable to find bound object"
		EndIf
	EndFunction
	
	Method getBinderHandle:Int()
		Return IrrScene_BMXIParticleAffector_getBinder(handle)
	EndMethod	
EndType

' ------------------------------------------------------------------------
' default implementation of IParticleAffector
' ------------------------------------------------------------------------
Type IParticleAffectorDefault Extends IParticleAffector		
	Method affect(now:Int,particlearray:Array_SParticle)
		IrrScene_IParticleAffector_affect(handle,now,particlearray.handle)
	EndMethod

	Method getType:Int() ' returns E_PARTICLE_AFFECTOR_TYPE
		Return IrrScene_IParticleAffector_getType(handle) 
	EndMethod
	
	Method serializeAttributes(out:IAttributes,options:SAttributeReadWriteOptions=Null)
		If Not options Then options=New SAttributeReadWriteOptions ' default handle to null
		IrrScene_IParticleAffector_serializeAttributes(handle,out.handle,options.handle)
	EndMethod
	
	Method deserializeAttributes(in:IAttributes,options:SAttributeReadWriteOptions=Null)
		If Not options Then options=New SAttributeReadWriteOptions ' default handle to null
		IrrScene_IParticleAffector_deserializeAttributes(handle,in.handle,options.handle)
	EndMethod	
EndType
