//------------------------------------------------------------------------------------
// irr::scene::IParticleSystemSceneNode wrappers (parent is ISceneNode)
//------------------------------------------------------------------------------------

extern "C" void IrrScene_IParticleSystemSceneNode_addAffector(IParticleSystemSceneNode* node,IParticleAffector *affector)
{
	if (node)
		node->addAffector(affector);
}

// you must call drop for this when done.
extern "C" IParticleEmitter* IrrScene_IParticleSystemSceneNode_createBoxEmitter(IParticleSystemSceneNode* node,
	aabbox3d<f32>* box=0,
	vector3df* direction=0,
	u32 minParticlesPerSecond=5, u32 maxParticlePerSecond=10,
	SColor* minStartColor=0,
	SColor* maxStartColor=0,
	u32 lifeTimeMin=2000, u32 lifeTimeMax=4000, s32 maxAngleDegrees=0)
{
	IParticleEmitter* pRetVal=0;

	bool delB=false,delD=false,delMin=false,delMax=false;
	if (box==0) {box=new aabbox3d<f32>(-10, 28,-10, 10, 30, 10); delB=true;}
	if (direction==0) {direction=new vector3df(0.0f, 0.03f, 0.0f); delD=true;}
	if (minStartColor==0) {minStartColor=new SColor(255, 0, 0, 0); delMin=true;}
	if (maxStartColor==0) {maxStartColor=new SColor(255, 255, 255, 255); delMax=true;}

	if (node)
		pRetVal=node->createBoxEmitter(*box,*direction,minParticlesPerSecond,maxParticlePerSecond,*minStartColor,*maxStartColor,lifeTimeMin,lifeTimeMax,maxAngleDegrees);

	if (delB) delete box;
	if (delD) delete direction;
	if (delMin) delete minStartColor;
	if (delMax) delete maxStartColor;

	return pRetVal;
}

// you must call drop for this when done.
extern "C" IParticleAffector* IrrScene_IParticleSystemSceneNode_createFadeOutParticleAffector(IParticleSystemSceneNode* node,SColor* targetColor=0,u32 timeNeededToFadeOut=1000)
{
	IParticleAffector* pRetVal=0;

	bool delT=false;
	if (!targetColor) {targetColor=new SColor(0, 0, 0, 0); delT=true;}

	if (node)
		pRetVal=node->createFadeOutParticleAffector(*targetColor,timeNeededToFadeOut);

	if (delT) delete targetColor;

	return pRetVal;
}

//  you must call drop for this when done.
extern "C" IParticleAffector* IrrScene_IParticleSystemSceneNode_createGravityAffector(IParticleSystemSceneNode* node,vector3df* gravity=0,u32 timeForceLost=1000)
{
	IParticleAffector* pRetVal=0;

	bool delG=false;
	if (!gravity) {gravity=new vector3df(0.0f,-0.03f, 0.0f); delG=true;}

	if (node)
		pRetVal=node->createGravityAffector(*gravity,timeForceLost);

	if (delG) delete gravity;

	return pRetVal;
}

// you must call drop for this when done.
extern "C" IParticleEmitter* IrrScene_IParticleSystemSceneNode_createPointEmitter(IParticleSystemSceneNode* node,
	vector3df* direction=0,
	u32 minParticlesPerSecond=5, u32 maxParticlePerSecond=10,
	SColor* minStartColor=0, 
	SColor* maxStartColor=0, 
	u32 lifeTimeMin=2000, u32 lifeTimeMax=4000, s32 maxAngleDegrees=0)
{
	IParticleEmitter* pRetVal=0;

	bool delD=false,delMin=false,delMax=false;
	if (direction==0) {direction=new vector3df(0.0f, 0.03f, 0.0f); delD=true;}
	if (minStartColor==0) {minStartColor=new SColor(255, 0, 0, 0); delMin=true;}
	if (maxStartColor==0) {maxStartColor=new SColor(255, 255, 255, 255); delMax=true;}

	if (node)
		pRetVal=node->createPointEmitter(*direction,minParticlesPerSecond,maxParticlePerSecond,*minStartColor,*maxStartColor,lifeTimeMin,lifeTimeMax,maxAngleDegrees);

	if (delD) delete direction;
	if (delMin) delete minStartColor;
	if (delMax) delete maxStartColor;

	return pRetVal;
}

extern "C" void IrrScene_IParticleSystemSceneNode_removeAllAffectors(IParticleSystemSceneNode* node)
{
	if (node)
		node->removeAllAffectors();
}

extern "C" void IrrScene_IParticleSystemSceneNode_setEmitter(IParticleSystemSceneNode* node,IParticleEmitter* emitter=0)
{
	if (node)
		node->setEmitter(emitter);
}

extern "C" void IrrScene_IParticleSystemSceneNode_setParticlesAreGlobal(IParticleSystemSceneNode* node,bool global)
{
	if (node)
		node->setParticlesAreGlobal(global);
}

extern "C" void IrrScene_IParticleSystemSceneNode_setParticleSize(IParticleSystemSceneNode* node,dimension2d<f32>* size=0)
{

	bool delS=false;
	if (!size) {size=new dimension2d<f32>(5.0f, 5.0f); delS=true;}

	if (node)
		node->setParticleSize(*size);
	
	if (delS) delete size;
}
