Public

' ------------------------------------------------------------------------------------
'  irr::scene::IQ3Shader wrappers
' ------------------------------------------------------------------------------------

' //! Hold the different Mesh Types used for getMesh
' enum eQ3MeshIndex
Const E_Q3_MESH_GEOMETRY:Int = 0
Const E_Q3_MESH_ITEMS:Int = 1
Const E_Q3_MESH_BILLBOARD:Int = 2
Const E_Q3_MESH_SIZE:Int = 3

Function Quake3_getAsFloat:Float(str:String,pos:Int Var)
	Return IrrScene_Quake3_getAsFloat( str, Varptr(pos) )
EndFunction

Function Quake3_getAsVector3df:Vector3df(str:String,pos:Int Var)
	Return Vector3df.createFromHandle(IrrScene_Quake3_getAsVector3df( str,Varptr(pos) ),True)
EndFunction
	
'  -------------------------------------------------------------
'  SVariable
'  -------------------------------------------------------------
Type SVariable Extends IrrBase Final

	Function createFromHandle:SVariable(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SVariable=New SVariable
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction
	
	Method getName:String()
		Return IrrScene_SVariable_getName(handle)
	EndMethod
	
	Method setName(name:String)
		IrrScene_SVariable_setName(handle,name)
	EndMethod
	
	Method getContent:String()
		Return IrrScene_SVariable_getContent(handle)
	EndMethod
	
	Method setContent(content:String)
		IrrScene_SVariable_setContent(handle,content)
	EndMethod
	
	Method clear()
		IrrScene_SVariable_clear(handle)
	EndMethod
	
	Method isValid:Int()
		Return IrrScene_SVariable_isValid(handle)
	EndMethod
	
	Method Less:Int(other:SVariable)
		Return IrrScene_SVariable_Less(handle,other.handle)
	EndMethod
	
EndType
		
'  -------------------------------------------------------------
'  SVarGroup
'  -------------------------------------------------------------
Type SVarGroup Extends IrrBase Final

	Function createFromHandle:SVarGroup(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SVarGroup=New SVarGroup
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method getIndex:Int(name:String)
		IrrScene_SVarGroup_getIndex(handle,name)
	EndMethod
	
	Method get:String(name:String)
		Return IrrScene_SVarGroup_get(handle,name)
	EndMethod
	
	Method isDefined:Int(name:String,content:String=Null)
		Return IrrScene_SVarGroup_isDefined(handle,name,content)
	EndMethod
	
	Method getVariables:SVariable[]()
		Local ptrs:Int Ptr	
		Local objs:SVariable[]=Null				

		Local size:Int=IrrScene_SVarGroup_getVariables(handle,Varptr(ptrs))
		If size>0 Then		
			objs=New SVariable[size]

			For Local i:Int=0 Until size
				objs[i]=SVariable.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		
		Return objs		
	EndMethod

EndType

'  -------------------------------------------------------------
'  SVarGroupList extends IUnknown
'  -------------------------------------------------------------
Type SVarGroupList Extends Tirr_IUnknown Final

	Function createFromHandle:SVarGroupList(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SVarGroupList=New SVarGroupList
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method drop:Int()
		Return IrrScene_SVarGroupList_drop(handle)	
	EndMethod
	
	Method getDebugName:String()
		Return IrrScene_SVarGroupList_getDebugName(handle)
	EndMethod
	
	Method grab()
		IrrScene_SVarGroupList_grab(handle)
	EndMethod
	
	Method getVariables:SVarGroup[]()
		Local ptrs:Int Ptr	
		Local objs:SVarGroup[]=Null				

		Local size:Int=IrrScene_SVarGroupList_getVariables(handle,Varptr(ptrs))
		If size>0 Then		
			objs=New SVarGroup[size]

			For Local i:Int=0 Until size
				objs[i]=SVarGroup.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		
		Return objs		
	EndMethod
EndType	

'  -------------------------------------------------------------
'  SShader
'  -------------------------------------------------------------
Type SShader Extends IrrBase Final

	Function createFromHandle:SShader(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SShader=New SShader
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method Less:Int(other:SShader)
		Return IrrScene_SShader_Less(handle,other.handle)
	EndMethod
	
	Method getID:Int()
		Return IrrScene_SShader_getID(handle)
	EndMethod
	
	Method getName:String()
		Return IrrScene_SShader_getName(handle)
	EndMethod
	
	Method getVarGroup:SVarGroupList()
		Return SVarGroupList.createFromHandle(IrrScene_SShader_getVarGroup(handle),False)
	EndMethod
	
	Method getGroup:SVarGroup(stage:Int)
		Return SVarGroup.createFromHandle(IrrScene_SShader_getGroup(handle,stage),False)
	EndMethod
	
EndType