Public

Type Vertex Extends IrrBase

	Method getPos:Vector3df() Abstract
	Method setPos(pos:Vector3df) Abstract

	Method getNormal:Vector3df() Abstract
	Method setNormal(normal:Vector3df) Abstract

	Method getTCoords:Vector2df() Abstract
	Method setTCoords(coords:Vector2df) Abstract
	
	Method GetColor:SColor() Abstract
	Method SetColor(color:SColor) Abstract

EndType


' ------------------------------------------------------------------------------------
' irr::video::S3DVertex wrappers
'
' WARNING: be careful about deleting pointers returned by some of these methods
' ------------------------------------------------------------------------------------

Type S3DVertex Extends Vertex Final

	Function createFromHandle:S3DVertex(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:S3DVertex=New S3DVertex
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrVideo_S3DVertex_del(handle)
	EndMethod	
	
	Function create:S3DVertex()
		Return createFromHandle(IrrVideo_S3DVertex_new())
	EndFunction	
	Function createFromVals:S3DVertex(x:Float,y:Float,z:Float,nx:Float,ny:Float,nz:Float,color:SColor,tu:Float,tv:Float)
		Return createFromHandle(IrrVideo_S3DVertex_newFromVals(x,y,z,nx,ny,nz,color.handle,tu,tv))
	EndFunction
	Function createFromVects:S3DVertex(pos:Vector3df,normal:Vector3df,color:SColor,tcoords:Vector2df)
		Return createFromHandle(IrrVideo_S3DVertex_newFromVects(pos.handle,normal.handle,color.handle,tcoords.handle))
	EndFunction

	Method EqEq:Int(other:S3DVertex)
		Return IrrVideo_S3DVertex_EqEq(handle,other.handle)
	EndMethod
	Method NotEq:Int(other:S3DVertex)
		Return IrrVideo_S3DVertex_NotEq(handle,other.handle)
	EndMethod

	Method getPos:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertex_getPos(handle),False)
	EndMethod
	Method setPos(pos:Vector3df)
		IrrVideo_S3DVertex_setPos(handle,pos.handle)
	EndMethod

	Method getNormal:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertex_getNormal(handle),False)
	EndMethod
	Method setNormal(normal:Vector3df)
		IrrVideo_S3DVertex_setNormal(handle,normal.handle)
	EndMethod

	Method getTCoords:Vector2df()
		' do not delete
		Return Vector2df.createFromHandle(IrrVideo_S3DVertex_getTCoords(handle),False)
	EndMethod
	Method setTCoords(coords:Vector2df)
		IrrVideo_S3DVertex_setTCoords(handle,coords.handle)
	EndMethod
	
	Method GetColor:SColor()
		' do not delete
		Return SColor.createFromHandle(IrrVideo_S3DVertex_getColor(handle),False)
	EndMethod
	Method SetColor(color:SColor)
		IrrVideo_S3DVertex_setColor(handle,color.handle)
	EndMethod
	
	Method getVertexInfo:Int(pos:Float[],norm:Float[],tcoords:Float[],color:Int[])
		Return IrrVideo_S3DVertex_getVertexInfo(handle,pos,norm,tcoords,color)
	EndMethod
		
EndType

' ------------------------------------------------------------------------------------
' irr::video::S3DVertex2TCoords wrappers
'
' WARNING: be careful about deleting pointers returned by some of these methods
' ------------------------------------------------------------------------------------
Type S3DVertex2TCoords Extends Vertex Final

	Function createFromHandle:S3DVertex2TCoords(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:S3DVertex2TCoords=New S3DVertex2TCoords
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrVideo_S3DVertex2TCoords_del(handle)
	EndMethod	
	
	Function create:S3DVertex2TCoords()
		Return createFromHandle(IrrVideo_S3DVertex2TCoords_new())
	EndFunction
	
	Function createFromValsWithDiffTexCoords:S3DVertex2TCoords(x:Float,y:Float,z:Float,color:SColor,tu:Float,tv:Float,tu2:Float,tv2:Float)
		Return createFromHandle(IrrVideo_S3DVertex2TCoords_newFromValsWithDiffTexCoords(x,y,z,color.handle,tu,tv,tu2,tv2))
	EndFunction
	
	Function createWithDiffTexCoords:S3DVertex2TCoords(pos:Vector3df,color:SColor,tcoords:Vector2df,tcoords2:Vector2df)
		Return createFromHandle(IrrVideo_S3DVertex2TCoords_newWithDiffTexCoords(pos.handle,color.handle,tcoords.handle,tcoords2.handle))
	EndFunction
	
	Function createFromValsWithNormalAndSameTexCoords:S3DVertex2TCoords(x:Float,y:Float,z:Float,nx:Float,ny:Float,nz:Float,color:SColor,tu:Float,tv:Float)
		Return createFromHandle(IrrVideo_S3DVertex2TCoords_newFromValsWithNormalAndSameTexCoords(x,y,z,nx,ny,nz,color.handle,tu,tv))
	EndFunction

	Function createWithWithNormalAndSameTexCoords:S3DVertex2TCoords(pos:Vector3df,normal:Vector3df,color:SColor,tcoords:Vector2df)
		Return createFromHandle(IrrVideo_S3DVertex2TCoords_newWithNormalAndSameTexCoords(pos.handle,normal.handle,color.handle,tcoords.handle))
	EndFunction

	Function createWithWithNormalAndDiffTexCoords:S3DVertex2TCoords(pos:Vector3df,normal:Vector3df,color:SColor,tcoords:Vector2df,tcoords2:Vector2df)
		Return createFromHandle(IrrVideo_S3DVertex2TCoords_newWithNormalAndDiffTexCoords(pos.handle,normal.handle,color.handle,tcoords.handle,tcoords2.handle))
	EndFunction

	Method getPos:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertex2TCoords_getPos(handle),False)
	EndMethod
	Method setPos(pos:Vector3df)
		IrrVideo_S3DVertex2TCoords_setPos(handle,pos.handle)
	EndMethod

	Method getNormal:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertex2TCoords_getNormal(handle),False)
	EndMethod
	Method setNormal(normal:Vector3df)
		IrrVideo_S3DVertex2TCoords_setNormal(handle,normal.handle)
	EndMethod

	Method getTCoords:Vector2df()
		' do not delete
		Return Vector2df.createFromHandle(IrrVideo_S3DVertex2TCoords_getTCoords(handle),False)
	EndMethod
	Method setTCoords(coords:Vector2df)
		IrrVideo_S3DVertex2TCoords_setTCoords(handle,coords.handle)
	EndMethod
	
	Method getTCoords2:Vector2df()
		' do not delete
		Return Vector2df.createFromHandle(IrrVideo_S3DVertex2TCoords_getTCoords2(handle),False)
	EndMethod
	Method setTCoords2(coords:Vector2df)
		IrrVideo_S3DVertex2TCoords_setTCoords2(handle,coords.handle)
	EndMethod

	Method GetColor:SColor()
		' do not delete
		Return SColor.createFromHandle(IrrVideo_S3DVertex2TCoords_getColor(handle),False)
	EndMethod
	Method SetColor(color:SColor)
		IrrVideo_S3DVertex2TCoords_setColor(handle,color.handle)
	EndMethod	

	Method EqEq:Int(other:S3DVertex2TCoords)
		Return IrrVideo_S3DVertex2TCoords_EqEq(handle,other.handle)
	EndMethod
	Method NotEq:Int(other:S3DVertex2TCoords)
		Return IrrVideo_S3DVertex2TCoords_NotEq(handle,other.handle)
	EndMethod

	Method getVertexInfo:Int(pos:Float[],norm:Float[],tcoords:Float[],color:Int[],tcoords2:Float[])
		Return IrrVideo_S3DVertex2TCoords_getVertexInfo(handle,pos,norm,tcoords,color,tcoords2)
	EndMethod

EndType


' ------------------------------------------------------------------------------------
'  irr::video::S3DVertexTangents wrappers
'
'  WARNING: be careful about deleting pointers returned by some of these methods
' ------------------------------------------------------------------------------------

Type S3DVertexTangents Extends Vertex Final

	Function createFromHandle:S3DVertexTangents(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:S3DVertexTangents=New S3DVertexTangents
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrVideo_S3DVertexTangents_del(handle)
	EndMethod	
	
	Function create:S3DVertexTangents()
		Return createFromHandle(IrrVideo_S3DVertexTangents_new())
	EndFunction
	Function createFromVals:S3DVertexTangents(x:Float,y:Float,z:Float)
		Return createFromHandle(IrrVideo_S3DVertexTangents_newFromVals(x,y,z))
	EndFunction
	Function createFromVects:S3DVertexTangents(pos:Vector3df,tcoords:Vector2df,color:SColor)
		Return createFromHandle(IrrVideo_S3DVertexTangents_newFromVects(pos.handle,tcoords.handle,color.handle))
	EndFunction

	Method getPos:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertexTangents_getPos(handle),False)
	EndMethod
	Method setPos(pos:Vector3df)
		IrrVideo_S3DVertexTangents_setPos(handle,pos.handle)
	EndMethod

	Method getNormal:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertexTangents_getNormal(handle),False)
	EndMethod
	Method setNormal(normal:Vector3df)
		IrrVideo_S3DVertexTangents_setNormal(handle,normal.handle)
	EndMethod

	Method getTCoords:Vector2df()
		' do not delete
		Return Vector2df.createFromHandle(IrrVideo_S3DVertexTangents_getTCoords(handle),False)
	EndMethod
	Method setTCoords(coords:Vector2df)
		IrrVideo_S3DVertexTangents_setTCoords(handle,coords.handle)
	EndMethod
	
	Method GetColor:SColor()
		' do not delete
		Return SColor.createFromHandle(IrrVideo_S3DVertexTangents_getColor(handle),False)
	EndMethod
	Method SetColor(color:SColor)
		IrrVideo_S3DVertexTangents_setColor(handle,color.handle)
	EndMethod	

	Method getTangent:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertexTangents_getTangent(handle),False)
	EndMethod
	Method setTangent(tangent:Vector3df)
		IrrVideo_S3DVertexTangents_setTangent(handle,tangent.handle)
	EndMethod

	Method getBinormal:Vector3df()
		' do not delete
		Return Vector3df.createFromHandle(IrrVideo_S3DVertexTangents_getBinormal(handle),False)
	EndMethod
	Method setBinormal(binormal:Vector3df)
		IrrVideo_S3DVertexTangents_setBinormal(handle,binormal.handle)
	EndMethod

	Method EqEq:Int(other:S3DVertexTangents)
		Return IrrVideo_S3DVertexTangents_EqEq(handle,other.handle)
	EndMethod
	Method NotEq:Int(other:S3DVertexTangents)
		Return IrrVideo_S3DVertexTangents_NotEq(handle,other.handle)
	EndMethod

	Method getVertexInfo:Int(pos:Float[],norm:Float[],tcoords:Float[],color:Int[],tangent:Float[],binormal:Float[])
		Return IrrVideo_S3DVertexTangents_getVertexInfo(handle,pos,norm,tcoords,color,tangent,binormal)
	EndMethod
	
EndType
