' ------------------------------------------------------------------------------------
'  irr::video::SMaterial wrappers
' 
'  WARNING: do Not Release handles obtained via the get() methods
' ------------------------------------------------------------------------------------
Public

' ! Maximum number of texture an SMaterial can have.
Const MATERIAL_MAX_TEXTURES:Int = 4
	
Type SMaterial Extends IrrBase Final

	Function create:SMaterial()
		Return createFromHandle(IrrVideo_SMaterial_new())
	EndFunction

	Function createFromMaterial:SMaterial(mat:SMaterial)
		Return createFromHandle(IrrVideo_SMaterial_newFromMaterial(mat.handle),True)
	EndFunction
	
	Function createFromHandle:SMaterial(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:SMaterial=New SMaterial
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrVideo_SMaterial_del(handle)
	EndMethod
	
	' returns E_MATERIAL_TYPE
	Method getMaterialType:Int()
		Return IrrVideo_SMaterial_getMaterialType(handle) 
	EndMethod
	
	Method setMaterialType(E_MATERIAL_TYPE:Int)
		IrrVideo_SMaterial_setMaterialType(handle,E_MATERIAL_TYPE)
	EndMethod

	Method getMaterialTypeParam:Float()
		Return IrrVideo_SMaterial_getMaterialTypeParam(handle) 
	EndMethod
	
	Method setMaterialTypeParam(typeparam:Float)
		IrrVideo_SMaterial_setMaterialTypeParam(handle,typeparam)
	EndMethod

	Method getMaterialTypeParam2:Float()
		Return IrrVideo_SMaterial_getMaterialTypeParam2(handle) 
	EndMethod
	
	Method setMaterialTypeParam2(typeparam:Float)
		IrrVideo_SMaterial_setMaterialTypeParam2(handle,typeparam)
	EndMethod

	Method setMaterial(idx:Int,newmat:SMaterial)
		IrrScene_ISceneNode_setMaterial(handle,idx,newmat.handle)
	EndMethod

	Method getAmbientColor:SColor()
		' do not delete
		Return SColor.createFromHandle(IrrVideo_SMaterial_getAmbientColor(handle),False)
	EndMethod
	
	Method getDiffuseColor:SColor()
		' do not delete
		Return SColor.createFromHandle(IrrVideo_SMaterial_getDiffuseColor(handle),False)
	EndMethod
	
	Method getEmissiveColor:SColor()
		' do not delete
		Return SColor.createFromHandle(IrrVideo_SMaterial_getEmissiveColor(handle),False)
	EndMethod
	
	Method getSpecularColor:SColor()
		' do not delete
		Return SColor.createFromHandle(IrrVideo_SMaterial_getSpecularColor(handle),False) 
	EndMethod
	
	Method getShininess:Float()
		Return IrrVideo_SMaterial_getShininess(handle)
	EndMethod
	
	Method setShininess(newshine:Float)
		IrrVideo_SMaterial_setShininess(handle,newshine)
	EndMethod

	Method getThickness:Float()
		Return IrrVideo_SMaterial_getThickness(handle)
	EndMethod
	
	Method setThickness(newthick:Float)
		IrrVideo_SMaterial_setThickness(handle,newthick)
	EndMethod

	Method getWireframe:Int()
		Return IrrVideo_SMaterial_getWireframe(handle)
	EndMethod
	
	Method setWireframe(bWireframe:Int)
		IrrVideo_SMaterial_setWireframe(handle,bWireframe)
	EndMethod
	
	Method getGouraudShading:Int()
		Return IrrVideo_SMaterial_getGouraudShading(handle)
	EndMethod

	Method setGouraudShading(bNewVal:Int)
		IrrVideo_SMaterial_setGouraudShading(handle,bNewVal)
	EndMethod

	Method getLighting:Int()
		Return IrrVideo_SMaterial_getLighting(handle)
	EndMethod
	
	Method setLighting(bNewVal:Int)
		IrrVideo_SMaterial_setLighting(handle,bNewVal)
	EndMethod
	
	Method getZBuffer:Int()
		Return IrrVideo_SMaterial_getZBuffer(handle)
	EndMethod
	
	Method setZBuffer(NewVal:Int)
		IrrVideo_SMaterial_setZBuffer(handle,NewVal)
	EndMethod
	
	Method getZWriteEnable:Int()
		Return IrrVideo_SMaterial_getZWriteEnable(handle)
	EndMethod
	
	Method setZWriteEnable(bNewVal:Int)
		IrrVideo_SMaterial_setZWriteEnable(handle,bNewVal)
	EndMethod
	
	Method getBackfaceCulling:Int()
		Return IrrVideo_SMaterial_getBackfaceCulling(handle)
	EndMethod
	
	Method setBackfaceCulling(bNewVal:Int)
		IrrVideo_SMaterial_setBackfaceCulling(handle,bNewVal)
	EndMethod
	
	Method getBilinearFilter:Int()
		Return IrrVideo_SMaterial_getBilinearFilter(handle)
	EndMethod
	
	Method setBilinearFilter(bNewVal:Int)
		IrrVideo_SMaterial_setBilinearFilter(handle,bNewVal)
	EndMethod
	
	Method getTrilinearFilter:Int()
		Return IrrVideo_SMaterial_getTrilinearFilter(handle)
	EndMethod
	
	Method setTrilinearFilter(bNewVal:Int)
		IrrVideo_SMaterial_setTrilinearFilter(handle,bNewVal)
	EndMethod
	
	Method getAnisotropicFilter:Int()
		Return IrrVideo_SMaterial_getAnisotropicFilter(handle)
	EndMethod
	
	Method setAnisotropicFilter(bNewVal:Int)
		IrrVideo_SMaterial_setAnisotropicFilter(handle,bNewVal)
	EndMethod
	
	Method getFogEnable:Int()
		Return IrrVideo_SMaterial_getFogEnable(handle)
	EndMethod
	
	Method setFogEnable(bNewVal:Int)
		IrrVideo_SMaterial_setFogEnable(handle,bNewVal)
	EndMethod
	
	Method getFlag:Int(E_MATERIAL_FLAG:Int)
		Return IrrVideo_SMaterial_getFlag(handle,E_MATERIAL_FLAG)
	EndMethod
	
	Method setFlag(E_MATERIAL_FLAG:Int,bNewVal:Int)
		IrrVideo_SMaterial_setFlag(handle,E_MATERIAL_FLAG,bNewVal)
	EndMethod
	
	Method getTexture:ITexture(idx:Int)
		' do not delete
		Return ITexture.createFromHandle(IrrVideo_SMaterial_getTexture(handle,idx),False)
	EndMethod
	
	Method setTexture(idx:Int,texture:ITexture)
		If texture=Null Then texture=New ITexture
		IrrVideo_SMaterial_setTexture(handle,idx,texture.handle)
	EndMethod

	Method getNormalizeNormals:Int()
		Return IrrVideo_SMaterial_getNormalizeNormals(handle)
	EndMethod

	Method setNormalizeNormals(bNewVal:Int)
		IrrVideo_SMaterial_setNormalizeNormals(handle,bNewVal)
	EndMethod

	Method getPointCloud:Int()
		Return IrrVideo_SMaterial_getPointCloud(handle)
	EndMethod
	
	Method setPointCloud(bPointCloud:Int)
		IrrVideo_SMaterial_setPointCloud(handle,bPointCloud)
	EndMethod

	Method SetEq:SMaterial(other:SMaterial)
		IrrVideo_SMaterial_SetEq(handle,other.handle)
		Return Self
	EndMethod

	Method NotEq:Int(other:SMaterial)
		Return IrrVideo_SMaterial_NotEq(handle,other.handle)
	EndMethod
	
	Method setTextureMatrix(idx:Int,mat:Matrix4)
		IrrVideo_SMaterial_setTextureMatrix(handle,idx,mat.handle)
	EndMethod
	
	Method getTextureMatrix:Matrix4(idx:Int)
		Return Matrix4.createFromHandle(IrrVideo_SMaterial_getTextureMatrix(handle,idx),False)
	EndMethod

	Method setTextureWrap(idx:Int,E_TEXTURE_CLAMP:Int)
		IrrVideo_SMaterial_setTextureWrap(handle,idx,E_TEXTURE_CLAMP)
	EndMethod
	
	Method getTextureWrap:Int(idx:Int)
		Return IrrVideo_SMaterial_getTextureWrap(handle,idx)
	EndMethod

EndType

